<?php

namespace Aerex\TaskwarriorPlugin\Taskwarrior\Commands;
use Aerex\TaskwarriorPlugin\Taskwarrior\Task;
use Aerex\TaskwarriorPlugin\Config;
use Symfony\Component\Process\Process;

class TodoStrategy implements IStrategy {

  public function __construct(TaskwarriorConfig $config){
    $this->config = $config;
  }

  public function add(Task $task){
    $cmd[] = $this->config->getTaskBin();
    $cmd[] = 'add';

    if($task->getDescription() != null){
      $cmd[] = sprintf('"%s"', (string)$task->getDescription());
    }

    if($task->getCategories() != null){
      $categories = implode(' +', (string)$task->getCategories());
      $cmd[] = $categories; 
    }

    if($task->getDue() != null){
      $cmd[] = sprintf("due:%s",$task->getDue('Y-m-dTH:i:s'));
    }

    if($task->getRecurrence() != null){
      $rrule = $task->getRecurrence()->getParts();
      $cmd[] = sprintf('recur:%s', $rrule['FREQ']);
      if(isset($rrule['UNTIL'])){
        $cmd[] = sprintf('until:%s', $rrule['UNTIL']);
      }

    }

    if($task->getStatus() != null){
      $cmd[] = sprintf('status:%s', (string)$task->getStatus());
    }

    return $this->executeCommand($cmd);
  }

  public function count($uuid){
    $cmd[] = $this->config->getTaskBin();
    $cmd[] = sprintf('%s count', $uuid);
    return $this->executeCommand($cmd);
  }




  private function executeCommand($command){
    $rcOptions = $this->config->getOptions();

    foreach ($rcOptions as $rcOption) {
      $command[] = $rcOption;
    }

    $cmdString = implode(' ', $command);
    echo $cmdString;
    $process = new Process($cmdString);

    $process->run();

    if(!$process->isSuccessful()){
      echo $process;
      throw new TaskwarriorCommandLineException($process);
    }

    // clear cmd queue
    $this->cmd = [];
    return $process->getOutput();

  }

}
